const background = {
  USER_ID_STORE: "userUUID",
  ACTIVE_STATUS_STORE: "isActive",
  KEYWORDS_STRING_STORE: "fwm_keywordsString",
  KEYWORDS_ARRAY_STORE: "fwm_keywordsArray",
  TAB_ACTIVE_STATUS: "tabActiveStatus",
  _cache: new Map(),
  _cacheTimeout: 2 * 60 * 1000,
  tabActiveStatus: new Map(),

  async setLocalStorage(key, value) {
    try {
      const oldValue = await this.getLocalStorage(key);
      if (oldValue === value) return;

      await chrome.storage.local.set({[key]: value});
      this._cache.set(key, {
        value,
        timestamp: Date.now()
      });

      await this._broadcastChange(key, value);
    } catch (error) {
      console.error('Error in setLocalStorage:', error);
    }
  },

  async getLocalStorage(key) {
    try {
      const cached = this._cache.get(key);
      if (cached && (Date.now() - cached.timestamp < this._cacheTimeout)) {
        return cached.value;
      }

      const result = await chrome.storage.local.get(key);
      this._cache.set(key, {
        value: result[key],
        timestamp: Date.now()
      });
      return result[key];
    } catch (error) {
      console.error('Error in getLocalStorage:', error);
      return null;
    }
  },

  async _broadcastChange(key, value) {
    try {
      // 只向当前活动标签页发送消息
      const [activeTab] = await chrome.tabs.query({
        active: true,
        currentWindow: true,
        url: ["http://*/*", "https://*/*"]
      });
      
      if (activeTab) {
        await chrome.tabs.sendMessage(activeTab.id, {
          opt: "event",
          event: "storageChange",
          args: {key, value}
        }).catch(() => {});
      }
    } catch (error) {
      console.warn('Broadcast change failed:', error);
    }
  },

  async setKeywordsString2(keywords, options = {}) {
    try {
      // 处理关键词,确保同一个词只出现一次
      const wordMap = new Map(); // 用于追踪每个词的最新状态
      const processedKeywords = keywords.reduce((acc, item, idx) => {
        if (item.status === 1 && item.data) {
          const words = item.data.split(/\s+/).filter(Boolean);
          words.forEach(word => {
            const lowerWord = word.toLowerCase();
            // 只保留每个词的最后一次出现
            wordMap.set(lowerWord, {
              colour: item.colour,
              words: word,
              categoryIndex: idx
            });
          });
        }
        return acc;
      }, []);

      // 将 Map 转换为数组
      const uniqueKeywords = Array.from(wordMap.values());

      // 保存数据
      await this.setLocalStorage(this.KEYWORDS_STRING_STORE, JSON.stringify(keywords));
      await this.setKeywords(uniqueKeywords);

      return uniqueKeywords;
    } catch (error) {
      console.error('Error in setKeywordsString2:', error);
      return null;
    }
  },

  async getKeywords() {
    return await this.getLocalStorage(this.KEYWORDS_ARRAY_STORE);
  },

  async setKeywords(keywords) {
    try {
      await this.setLocalStorage(this.KEYWORDS_ARRAY_STORE, keywords);
    } catch (error) {
      console.error('Error in setKeywords:', error);
    }
  },

  async setTabActiveStatus(tabId, status) {
    try {
      this.tabActiveStatus.set(tabId, status);
      await this.setLocalStorage(`${this.TAB_ACTIVE_STATUS}_${tabId}`, status);
      await this.setLocalStorage(this.ACTIVE_STATUS_STORE, status ? "true" : "false");
    } catch (error) {
      console.error('Error in setTabActiveStatus:', error);
    }
  },

  async getTabActiveStatus(tabId) {
    try {
      // 先获取全局状态
      const globalStatus = await this.getActiveStatus();
      if (!globalStatus) {
        return false;  // 如果全局状态是关闭的，标签页状态也一定是关闭的
      }

      // 如果全局状态是开启的，再检查标签页状态
      if (this.tabActiveStatus.has(tabId)) {
        return this.tabActiveStatus.get(tabId);
      }
      
      const status = await this.getLocalStorage(`${this.TAB_ACTIVE_STATUS}_${tabId}`);
      if (status !== null && status !== undefined) {
        this.tabActiveStatus.set(tabId, status);
        return status;
      }

      // 如果没有找到标签页状态，使用全局状态
      this.tabActiveStatus.set(tabId, globalStatus);
      await this.setLocalStorage(`${this.TAB_ACTIVE_STATUS}_${tabId}`, globalStatus);
      
      return globalStatus;
    } catch (error) {
      console.error('Error in getTabActiveStatus:', error);
      return false;
    }
  },

  generateUUID() {
    const lut = Array(256).fill().map((_, i) => (i < 16 ? '0' : '') + (i).toString(16));
    return () => {
      const d0 = Math.random() * 0xffffffff | 0;
      const d1 = Math.random() * 0xffffffff | 0;
      const d2 = Math.random() * 0xffffffff | 0;
      const d3 = Math.random() * 0xffffffff | 0;
      return `${lut[d0 & 0xff] + lut[d0 >> 8 & 0xff] + lut[d0 >> 16 & 0xff] + lut[d0 >> 24 & 0xff]}-${
        lut[d1 & 0xff]}${lut[d1 >> 8 & 0xff]}-${lut[d1 >> 16 & 0x0f | 0x40]}${lut[d1 >> 24 & 0xff]}-${
        lut[d2 & 0x3f | 0x80]}${lut[d2 >> 8 & 0xff]}-${lut[d2 >> 16 & 0xff]}${lut[d2 >> 24 & 0xff]}${
        lut[d3 & 0xff]}${lut[d3 >> 8 & 0xff]}${lut[d3 >> 16 & 0xff]}${lut[d3 >> 24 & 0xff]}`;
    };
  },

  async getUserId() {
    try {
      let userUUID = await this.getLocalStorage(this.USER_ID_STORE);
      if (!userUUID) {
        userUUID = this.generateUUID();
        await this.setLocalStorage(this.USER_ID_STORE, userUUID);
      }
      return userUUID;
    } catch (error) {
      console.error('Error in getUserId:', error);
      return null;
    }
  },

  async getActiveStatus() {
    try {
      const status = await this.getLocalStorage(this.ACTIVE_STATUS_STORE);
      return status !== "false";
    } catch (error) {
      console.error('Error in getActiveStatus:', error);
      return true;
    }
  },

  async setActiveStatus(status) {
    try {
      // 只保存全局状态
      await this.setLocalStorage(this.ACTIVE_STATUS_STORE, status ? "true": "false");
      
      // 获取当前活动标签页
      const [activeTab] = await chrome.tabs.query({
        active: true,
        currentWindow: true,
        url: ["http://*/*", "https://*/*"]
      });
      
      // 只更新当前标签页的状态
      if (activeTab) {
        this.tabActiveStatus.set(activeTab.id, status);
        await this.setLocalStorage(`${this.TAB_ACTIVE_STATUS}_${activeTab.id}`, status);
      }
    } catch (error) {
      console.error('Error in setActiveStatus:', error);
    }
  },

  async getKeywordsString() {
    try {
      return await this.getLocalStorage(this.KEYWORDS_STRING_STORE) || "";
    } catch (error) {
      console.error('Error in getKeywordsString:', error);
      return "";
    }
  },

  async getKeywordsString2() {
    try {
      const data = await this.getLocalStorage(this.KEYWORDS_STRING_STORE);
      return data ? JSON.parse(data) : [];
    } catch (error) {
      console.error('Error in getKeywordsString2:', error);
      return [];
    }
  },

  async reapplyHighlights() {
    try {
      const [activeTab] = await chrome.tabs.query({
        active: true,
        currentWindow: true,
        url: ["http://*/*", "https://*/*"]
      });
      
      if (activeTab) {
        await chrome.tabs.sendMessage(activeTab.id, {
          opt: "event",
          event: "reapplyHighlights"
        }).catch(() => {});
      }
    } catch (error) {
      console.warn('Reapply highlights failed:', error);
    }
  }
};

// 处理消息
chrome.runtime.onMessage.addListener((message, sender, sendResponse) => {
  if (message.opt === "rpc" && background[message.func]) {
    (async () => {
      try {
        const result = await Promise.race([
          background[message.func].apply(background, message.args),
          new Promise((_, reject) => {
            const timeout = message.func === 'getActiveStatus' ? 3000 : 5000;  // 单位:毫秒
            setTimeout(() => reject(new Error('Timeout')), timeout);
          })
        ]);
        sendResponse(result);
      } catch (error) {
        console.error('Error in message handler:', error);
        sendResponse(null);
      }
    })();
    return true;
  }
});

// 标签页关闭时清理
chrome.tabs.onRemoved.addListener((tabId) => {
  background.tabActiveStatus.delete(tabId);
  chrome.storage.local.remove(`${background.TAB_ACTIVE_STATUS}_${tabId}`);
});

// 标签页更新监听
chrome.tabs.onUpdated.addListener(async (tabId, changeInfo, tab) => {
  if (changeInfo.status === 'complete' && 
    tab.url?.match(/^(http|https|file):\/\//)) {
    try {
      const status = await Promise.race([
        background.getTabActiveStatus(tabId),
        new Promise((_, reject) => 
          setTimeout(() => reject(new Error('Timeout')), 300)
        )
      ]);
      background.tabActiveStatus.set(tabId, status);
    } catch (error) {
      console.error('Error updating tab status:', error);
    }
  }
});


