// 启动初始化
// if (document.readyState === 'loading') {
//     document.addEventListener('DOMContentLoaded', () => {
//         // 延迟初始化,让页面先加载完成
//         setTimeout(initialize, 100); 
//     });
// } else {
//     // 页面已经加载完成,也适当延迟初始化
//     setTimeout(initialize, 50);
// }
if (document.readyState === 'loading') {
    document.addEventListener('DOMContentLoaded', initialize);
} else {
    initialize();
}


// 初始化函数
async function initialize() {
    try {
        const [tabActive, keywords] = await Promise.all([
            chrome.runtime.sendMessage({
                opt: "rpc", 
                func: "getTabActiveStatus",
                args: [chrome.runtime.id]
            }),
            chrome.runtime.sendMessage({
                opt: "rpc",
                func: "getKeywords"  
            })
        ]);

        window.tabActive = tabActive;
        window.keywords = keywords || [];

        if (window.tabActive && window.keywords.length) {
            window.highlighter.highlight(document.body, window.keywords, true);
        }

        setupObservers();
        setupRouteObserver();

        // 消息监听优化
        chrome.runtime.onMessage.addListener((message) => {
            if (message.opt === "event") {
                switch(message.event) {
                    case "storageChange":
                        handleStorageChange(message);
                        break;
                    case "clearHighlights":
                        window.highlighter.clearHighlighted(document.body);
                        break;
                }
            }
        });

    } catch (error) {
        console.error('Initialization error:', error);
        setTimeout(initialize, 200);
    }
}

// 修改 observer 定义和配置
const observer = new MutationObserver((mutations) => {
    if (!window.tabActive || !window.highlighter || 
        !Array.isArray(window.keywords) || !window.keywords.length) return;

    const nodesToProcess = new Set();

    mutations.forEach(mutation => {
        if (mutation.type === 'childList') {
            mutation.addedNodes.forEach(node => {
                if (node.nodeType === Node.ELEMENT_NODE) {
                    nodesToProcess.add(node);
                }
            });
        }
        else if (mutation.type === 'characterData') {
            const target = mutation.target.parentNode;
            if (target) {
                nodesToProcess.add(target);
            }
        }
    });

    if (nodesToProcess.size > 0) {
        requestAnimationFrame(() => {
            nodesToProcess.forEach(node => {
                window.highlighter.highlight(node, window.keywords, true);
            });
        });
    }
});

// 修改 observer 配置
function setupObservers() {
    observer.observe(document.body, {
        childList: true,
        subtree: true,
        characterData: true,
        characterDataOldValue: true,
        attributes: true,
        attributeFilter: ['class', 'style', 'hidden']
    });
}

// 修改存储变化处理函数
function handleStorageChange(message) {
    if (message.args.key === "isActive") {
        window.tabActive = message.args.value;
        if (!window.tabActive) {
            window.highlighter.clearHighlighted(document.body);
        } else if (window.keywords?.length) {
            window.highlighter.clearHighlighted(document.body);
            window.highlighter.highlight(document.body, window.keywords, true);
        }
    }
    if (message.args.key === "fwm_keywordsArray") {
        if (window.tabActive) {
            window.keywords = message.args.value || [];
            window.highlighter.clearHighlighted(document.body);
            window.highlighter.highlight(document.body, window.keywords, true);
        }
    }
}

// 添加路由变化监听
function setupRouteObserver() {
    let lastUrl = location.href;
    let routeChangeTimer = null;

    const routeObserver = new MutationObserver(() => {
        const url = location.href;
        if (url !== lastUrl) {
            lastUrl = url;
            clearTimeout(routeChangeTimer);
            routeChangeTimer = setTimeout(() => {
                if (window.tabActive && window.keywords?.length) {
                    window.highlighter.highlight(document.body, window.keywords, true);
                }
            }, 200);
        }
    });

    routeObserver.observe(document.body, {
        childList: true,
        subtree: true
    });
}

// 懒加载管理页面相关资源
async function loadManageResources() {
    if (location.pathname === '/manage.html') {
        await Promise.all([
            import('./manage.js'),
            loadCSS('css/element-plus.css')
        ]);
    }
}

function loadCSS(href) {
    return new Promise((resolve) => {
        const link = document.createElement('link');
        link.rel = 'stylesheet';
        link.href = href;
        link.onload = resolve;
        document.head.appendChild(link);
    });
}
